searchData={"items":[{"type":"module","title":"OpenAI.Responses.Helpers.ImageHelpers","doc":"Helper functions for creating structured messages with images for the OpenAI Responses API.","ref":"OpenAI.Responses.Helpers.ImageHelpers.html"},{"type":"function","title":"OpenAI.Responses.Helpers.ImageHelpers.create_message_with_images/3","doc":"Creates a structured message with text and images for vision-based models.","ref":"OpenAI.Responses.Helpers.ImageHelpers.html#create_message_with_images/3"},{"type":"function","title":"Parameters - OpenAI.Responses.Helpers.ImageHelpers.create_message_with_images/3","doc":"* `text` - The text prompt to include in the message\n  * `images` - Optional image(s) to include, can be:\n    * A single image (URL string, file path, or {url/path, detail} tuple)\n    * A list of images (URL strings, file paths, or {url/path, detail} tuples)\n  * `opts` - Options for the message:\n    * `:detail` - Default detail level for all images: \"low\", \"high\", or \"auto\" (default)\n    * `:role` - Message role, defaults to \"user\"","ref":"OpenAI.Responses.Helpers.ImageHelpers.html#create_message_with_images/3-parameters"},{"type":"function","title":"Image Specification - OpenAI.Responses.Helpers.ImageHelpers.create_message_with_images/3","doc":"Images can be specified in several ways:\n  * URL string: \"https://example.com/image.jpg\"\n  * Local file path: \"/path/to/image.jpg\"\n  * Tuple with detail level: {\"https://example.com/image.jpg\", \"high\"}\n  * Tuple with detail level: {\"/path/to/image.jpg\", \"low\"}\n\nLocal image files will be automatically encoded as Base64 data URLs.\nSupported image formats: .png, .jpeg, .jpg, .webp, .gif","ref":"OpenAI.Responses.Helpers.ImageHelpers.html#create_message_with_images/3-image-specification"},{"type":"function","title":"Examples - OpenAI.Responses.Helpers.ImageHelpers.create_message_with_images/3","doc":"# Simple text with one image URL\n    iex> create_message_with_images(\"What is in this image?\", \"https://example.com/image.jpg\")\n\n    # Text with a local image file\n    iex> create_message_with_images(\"Describe this image\", \"/path/to/image.jpg\")\n\n    # Using high detail for all images\n    iex> create_message_with_images(\"Analyze in detail\", [\"img1.jpg\", \"img2.jpg\"], detail: \"high\")\n\n    # Mixed image sources with specific detail levels\n    iex> create_message_with_images(\n    ...>   \"Compare these\", \n    ...>   [\n    ...>     {\"img1.jpg\", \"low\"},\n    ...>     {\"https://example.com/img2.jpg\", \"high\"}\n    ...>   ]\n    ...> )\n\n    # Text-only message\n    iex> create_message_with_images(\"Just a text question\")","ref":"OpenAI.Responses.Helpers.ImageHelpers.html#create_message_with_images/3-examples"},{"type":"module","title":"OpenAI.Responses","doc":"Client for the OpenAI Responses API.\n\nThis module provides functions to interact with OpenAI's Responses API,\nallowing you to create, retrieve, and manage AI-generated responses.","ref":"OpenAI.Responses.html"},{"type":"module","title":"Examples - OpenAI.Responses","doc":"# Create a simple text response\n    {:ok, response} = OpenAI.Responses.create(\n      model: \"gpt-4.1\",\n      input: \"Write a haiku about programming\"\n    )\n\n    # Extract the text from the response\n    text = OpenAI.Responses.Helpers.output_text(response)\n\n    # Create a response with tools and options\n    {:ok, response} = OpenAI.Responses.create(\n      model: \"gpt-4.1\",\n      input: \"What's the weather like in Paris?\",\n      tools: [%{type: \"web_search_preview\"}],\n      temperature: 0.7\n    )\n\n    # Stream a response\n    stream = OpenAI.Responses.create_stream(\n      model: \"gpt-4.1\",\n      input: \"Tell me a story\"\n    )\n    Enum.each(stream, fn event -> IO.inspect(event) end)","ref":"OpenAI.Responses.html#module-examples"},{"type":"function","title":"OpenAI.Responses.collect_stream/1","doc":"Collects a complete response from a streaming response.\n\nThis is a convenience function that consumes a stream and returns a complete response,\nsimilar to what would be returned by the non-streaming API. All events are processed\nand combined into a final response object.","ref":"OpenAI.Responses.html#collect_stream/1"},{"type":"function","title":"Parameters - OpenAI.Responses.collect_stream/1","doc":"* `stream` - The stream from OpenAI.Responses.stream/1","ref":"OpenAI.Responses.html#collect_stream/1-parameters"},{"type":"function","title":"Returns - OpenAI.Responses.collect_stream/1","doc":"* The complete response map","ref":"OpenAI.Responses.html#collect_stream/1-returns"},{"type":"function","title":"Examples - OpenAI.Responses.collect_stream/1","doc":"# Get a streaming response\n    stream = OpenAI.Responses.stream(model: \"gpt-4.1\", input: \"Tell me a story\")\n\n    # Collect all events into a single response object\n    response = OpenAI.Responses.collect_stream(stream)\n\n    # Process the complete response\n    text = OpenAI.Responses.Helpers.output_text(response)\n    IO.puts(text)","ref":"OpenAI.Responses.html#collect_stream/1-examples"},{"type":"function","title":"OpenAI.Responses.create/1","doc":"Creates a new response.","ref":"OpenAI.Responses.html#create/1"},{"type":"function","title":"Parameters - OpenAI.Responses.create/1","doc":"* `opts` - Keyword list containing the request parameters:\n    * `:client` - The parameters to initialize a custom client. See `Client.new/1` for more details.\n    * `:model` - The model ID to use (e.g., \"gpt-4.1\"). *This option is required.*\n    * `:input` - The text prompt or structured input message. *This option is required.*\n    * `:tools` - List of tools to make available to the model\n    * `:instructions` - System instructions for the model\n    * `:temperature` - Sampling temperature (0.0 to 2.0)\n    * `:max_output_tokens` - Maximum number of tokens to generate\n    * `:stream` - Whether to stream the response (use `stream/1` for proper streaming)\n    * `:previous_response_id` - ID of a previous response for continuation\n    * All other parameters supported by the API","ref":"OpenAI.Responses.html#create/1-parameters"},{"type":"function","title":"Returns - OpenAI.Responses.create/1","doc":"* `{:ok, response}` - On success, returns the response\n  * `{:error, error}` - On failure, potentially including `KeyError` if `:model` or `:input` are missing.","ref":"OpenAI.Responses.html#create/1-returns"},{"type":"function","title":"OpenAI.Responses.delete/2","doc":"Deletes a specific response by ID.","ref":"OpenAI.Responses.html#delete/2"},{"type":"function","title":"Parameters - OpenAI.Responses.delete/2","doc":"* `response_id` - The ID of the response to delete\n  * `opts` - Optional parameters for the request","ref":"OpenAI.Responses.html#delete/2-parameters"},{"type":"function","title":"Returns - OpenAI.Responses.delete/2","doc":"* `{:ok, result}` - On success, returns deletion confirmation\n  * `{:error, error}` - On failure","ref":"OpenAI.Responses.html#delete/2-returns"},{"type":"function","title":"OpenAI.Responses.get/2","doc":"Retrieves a specific response by ID.","ref":"OpenAI.Responses.html#get/2"},{"type":"function","title":"Parameters - OpenAI.Responses.get/2","doc":"* `response_id` - The ID of the response to retrieve\n  * `opts` - Optional parameters for the request\n    * `:include` - Additional data to include in the response","ref":"OpenAI.Responses.html#get/2-parameters"},{"type":"function","title":"Returns - OpenAI.Responses.get/2","doc":"* `{:ok, response}` - On success, returns the response\n  * `{:error, error}` - On failure","ref":"OpenAI.Responses.html#get/2-returns"},{"type":"function","title":"OpenAI.Responses.list_input_items/2","doc":"Lists input items for a specific response.","ref":"OpenAI.Responses.html#list_input_items/2"},{"type":"function","title":"Parameters - OpenAI.Responses.list_input_items/2","doc":"* `response_id` - The ID of the response\n  * `opts` - Optional parameters for the request\n    * `:before` - List input items before this ID\n    * `:after` - List input items after this ID\n    * `:limit` - Number of objects to return (1-100)\n    * `:order` - Sort order (\"asc\" or \"desc\")","ref":"OpenAI.Responses.html#list_input_items/2-parameters"},{"type":"function","title":"Returns - OpenAI.Responses.list_input_items/2","doc":"* `{:ok, items}` - On success, returns the input items\n  * `{:error, error}` - On failure","ref":"OpenAI.Responses.html#list_input_items/2-returns"},{"type":"function","title":"OpenAI.Responses.parse/2","doc":"Creates a response with structured output.\n\nThis function is similar to `create/1` but automatically parses the response\naccording to the provided schema and returns the parsed data.","ref":"OpenAI.Responses.html#parse/2"},{"type":"function","title":"Parameters - OpenAI.Responses.parse/2","doc":"* `schema` - The schema definition for structured output\n  * `opts` - Keyword list containing the request parameters:\n    * `:model` - The model ID to use (e.g., \"gpt-4.1\"). *This option is required.*\n    * `:input` - The text prompt or structured input message. *This option is required.*\n    * `:schema_name` - Optional name for the schema (default: \"data\")\n    * `:strict` - Whether the output must conform strictly to the schema (default: true)\n    * All other options supported by `create/1`","ref":"OpenAI.Responses.html#parse/2-parameters"},{"type":"function","title":"Returns - OpenAI.Responses.parse/2","doc":"* `{:ok, result_map}` - On success, returns a map containing:\n    * `:parsed` - The parsed data according to the schema.\n    * `:raw_response` - The complete, raw response map received from the API.\n    * `:token_usage` - The token usage map from the API response (e.g., `%{ \"input_tokens\" => 10, \"output_tokens\" => 50 }`), or `nil` if not present.\n  * `{:error, error}` - On failure, potentially including `KeyError` if `:model` or `:input` are missing.","ref":"OpenAI.Responses.html#parse/2-returns"},{"type":"function","title":"Examples - OpenAI.Responses.parse/2","doc":"# Define a schema\n    calendar_event_schema = OpenAI.Responses.Schema.object(%{\n      name: :string,\n      date: :string,\n      participants: {:array, :string}\n    })\n\n    # Create a response with structured output\n    {:ok, result} = OpenAI.Responses.parse(\n      calendar_event_schema,\n      model: \"gpt-4.1\",\n      input: \"Alice and Bob are going to a science fair on Friday.\",\n      schema_name: \"event\"\n    )\n\n    # Access the parsed data and metadata\n    IO.puts(\"Event: #{result.parsed[\"name\"]} on #{result.parsed[\"date\"]}\")\n    IO.puts(\"Participants: #{Enum.join(result.parsed[\"participants\"], \", \")}\")\n    IO.inspect(result.token_usage, label: \"Token Usage\")","ref":"OpenAI.Responses.html#parse/2-examples"},{"type":"function","title":"OpenAI.Responses.stream/1","doc":"Creates a streaming response and returns a proper Enumerable stream of events.\n\nThis function returns a stream that yields individual events as they arrive from the API,\nmaking it suitable for real-time processing of responses.","ref":"OpenAI.Responses.html#stream/1"},{"type":"function","title":"Parameters - OpenAI.Responses.stream/1","doc":"* `opts` - Keyword list containing the request parameters:\n    * `:model` - The model ID to use (e.g., \"gpt-4.1\"). *This option is required.*\n    * `:input` - The text prompt or structured input message. *This option is required.*\n    * Other options supported by `create/1`","ref":"OpenAI.Responses.html#stream/1-parameters"},{"type":"function","title":"Examples - OpenAI.Responses.stream/1","doc":"# Print each event as it arrives\n    stream = OpenAI.Responses.stream(model: \"gpt-4.1\", input: \"Tell me a story\")\n    Enum.each(stream, &IO.inspect/1)\n\n    # Process text deltas in real-time\n    stream = OpenAI.Responses.stream(model: \"gpt-4.1\", input: \"Tell me a story\")\n    text_stream = OpenAI.Responses.Stream.text_deltas(stream)\n\n    # This preserves streaming behavior (one chunk at a time)\n    text_stream\n    |> Stream.each(fn delta ->\n      IO.write(delta)\n    end)\n    |> Stream.run()","ref":"OpenAI.Responses.html#stream/1-examples"},{"type":"function","title":"Returns - OpenAI.Responses.stream/1","doc":"* An Enumerable stream that yields events as they arrive. Will raise `KeyError` during enumeration if `:model` or `:input` are missing.","ref":"OpenAI.Responses.html#stream/1-returns"},{"type":"function","title":"OpenAI.Responses.text_deltas/1","doc":"Extracts text deltas from a streaming response.\n\nThis is a convenience function that returns a stream of text chunks as they arrive,\nuseful for real-time display of model outputs. The function ensures text is not duplicated\nin the final output.","ref":"OpenAI.Responses.html#text_deltas/1"},{"type":"function","title":"Parameters - OpenAI.Responses.text_deltas/1","doc":"* `stream` - The stream from OpenAI.Responses.stream/1","ref":"OpenAI.Responses.html#text_deltas/1-parameters"},{"type":"function","title":"Returns - OpenAI.Responses.text_deltas/1","doc":"* A stream of text deltas","ref":"OpenAI.Responses.html#text_deltas/1-returns"},{"type":"function","title":"Examples - OpenAI.Responses.text_deltas/1","doc":"stream = OpenAI.Responses.stream(model: \"gpt-4.1\", input: \"Tell me a story\")\n    text_stream = OpenAI.Responses.text_deltas(stream)\n\n    # Print text deltas as they arrive (real-time output)\n    text_stream\n    |> Stream.each(fn delta ->\n      IO.write(delta)\n    end)\n    |> Stream.run()\n    IO.puts(\"\")   # Add a newline at the end\n\n    # Create a typing effect\n    stream = OpenAI.Responses.stream(model: \"gpt-4.1\", input: \"Tell me a story\")\n    text_stream = OpenAI.Responses.text_deltas(stream)\n\n    text_stream\n    |> Stream.each(fn delta ->\n      IO.write(delta)\n      Process.sleep(10)  # Add delay for typing effect\n    end)\n    |> Stream.run()","ref":"OpenAI.Responses.html#text_deltas/1-examples"},{"type":"module","title":"OpenAI.Responses.Schema","doc":"Utilities for defining structured output schemas for OpenAI responses.\n\nThis module provides a simple, idiomatic Elixir way to define JSON schemas\nthat can be used with the OpenAI Responses API to ensure structured outputs.","ref":"OpenAI.Responses.Schema.html"},{"type":"module","title":"Simple Usage - OpenAI.Responses.Schema","doc":"For simple types without constraints, you can use atoms directly:\n\n    # Using atoms for simple types\n    schema = OpenAI.Responses.Schema.object(%{\n      name: :string,\n      age: :integer,\n      is_active: :boolean,\n      scores: {:array, :number}\n    })\n\nFor types with constraints or additional options, use the corresponding functions:\n\n    # Using functions for types with constraints\n    schema = OpenAI.Responses.Schema.object(%{\n      username: OpenAI.Responses.Schema.string(min_length: 3),\n      rating: OpenAI.Responses.Schema.number(minimum: 0, maximum: 5)\n    })","ref":"OpenAI.Responses.Schema.html#module-simple-usage"},{"type":"function","title":"OpenAI.Responses.Schema.array/2","doc":"Creates an array schema definition.","ref":"OpenAI.Responses.Schema.html#array/2"},{"type":"function","title":"Parameters - OpenAI.Responses.Schema.array/2","doc":"* `items` - The schema for items in the array. Can be:\n    * An atom (`:string`, `:number`, `:integer`, `:boolean`) for simple types\n    * A schema map created by other Schema functions\n  * `opts` - Optional parameters\n    * `:min_items` - Minimum number of items\n    * `:max_items` - Maximum number of items","ref":"OpenAI.Responses.Schema.html#array/2-parameters"},{"type":"function","title":"Examples - OpenAI.Responses.Schema.array/2","doc":"# Array of strings (simple)\n    participants_schema = {:array, :string}\n    # or\n    participants_schema = OpenAI.Responses.Schema.array(:string)\n\n    # Array of strings with constraints\n    tags_schema = OpenAI.Responses.Schema.array(:string, min_items: 1, max_items: 5)\n\n    # Array of objects\n    steps_schema = OpenAI.Responses.Schema.array(\n      OpenAI.Responses.Schema.object(%{\n        explanation: :string,\n        output: :string\n      })\n    )","ref":"OpenAI.Responses.Schema.html#array/2-examples"},{"type":"function","title":"OpenAI.Responses.Schema.boolean/0","doc":"Creates a boolean schema definition.\n\nFor a simple boolean, you can use the atom `:boolean` directly\nin `object/2` or `array/2` instead of calling this function.","ref":"OpenAI.Responses.Schema.html#boolean/0"},{"type":"function","title":"Examples - OpenAI.Responses.Schema.boolean/0","doc":"# Boolean field (in object definition)\n    is_active_field = :boolean\n\n    # Or using the function\n    is_active_field = OpenAI.Responses.Schema.boolean()","ref":"OpenAI.Responses.Schema.html#boolean/0-examples"},{"type":"function","title":"OpenAI.Responses.Schema.integer/1","doc":"Creates an integer schema definition with optional constraints.\n\nFor a simple integer without constraints, you can use the atom `:integer` directly\nin `object/2` or `array/2` instead of calling this function.","ref":"OpenAI.Responses.Schema.html#integer/1"},{"type":"function","title":"Parameters - OpenAI.Responses.Schema.integer/1","doc":"* `opts` - Optional parameters\n    * `:minimum` - Minimum value\n    * `:maximum` - Maximum value\n    * `:exclusive_minimum` - Whether minimum is exclusive\n    * `:exclusive_maximum` - Whether maximum is exclusive\n    * `:multiple_of` - Integer must be multiple of this value","ref":"OpenAI.Responses.Schema.html#integer/1-parameters"},{"type":"function","title":"Examples - OpenAI.Responses.Schema.integer/1","doc":"# Simple integer (in object definition)\n    count_field = :integer\n\n    # Integer with range\n    age_field = OpenAI.Responses.Schema.integer(\n      minimum: 0,\n      maximum: 120\n    )\n\n    # Positive integer\n    quantity_field = OpenAI.Responses.Schema.integer(\n      minimum: 1\n    )","ref":"OpenAI.Responses.Schema.html#integer/1-examples"},{"type":"function","title":"OpenAI.Responses.Schema.nullable/1","doc":"Creates a nullable schema definition.","ref":"OpenAI.Responses.Schema.html#nullable/1"},{"type":"function","title":"Parameters - OpenAI.Responses.Schema.nullable/1","doc":"* `schema` - The base schema or atom type","ref":"OpenAI.Responses.Schema.html#nullable/1-parameters"},{"type":"function","title":"Examples - OpenAI.Responses.Schema.nullable/1","doc":"# Nullable string\n    middle_name_field = OpenAI.Responses.Schema.nullable(:string)\n\n    # Nullable object\n    address_field = OpenAI.Responses.Schema.nullable(\n      OpenAI.Responses.Schema.object(%{\n        street: :string,\n        city: :string,\n        zip: :string\n      })\n    )","ref":"OpenAI.Responses.Schema.html#nullable/1-examples"},{"type":"function","title":"OpenAI.Responses.Schema.number/1","doc":"Creates a number schema definition with optional constraints.\n\nFor a simple number without constraints, you can use the atom `:number` directly\nin `object/2` or `array/2` instead of calling this function.","ref":"OpenAI.Responses.Schema.html#number/1"},{"type":"function","title":"Parameters - OpenAI.Responses.Schema.number/1","doc":"* `opts` - Optional parameters\n    * `:minimum` - Minimum value\n    * `:maximum` - Maximum value\n    * `:exclusive_minimum` - Whether minimum is exclusive\n    * `:exclusive_maximum` - Whether maximum is exclusive\n    * `:multiple_of` - Number must be multiple of this value","ref":"OpenAI.Responses.Schema.html#number/1-parameters"},{"type":"function","title":"Examples - OpenAI.Responses.Schema.number/1","doc":"# Simple number (in object definition)\n    price_field = :number\n\n    # Number with range\n    rating_field = OpenAI.Responses.Schema.number(\n      minimum: 0,\n      maximum: 5\n    )\n\n    # Positive number\n    quantity_field = OpenAI.Responses.Schema.number(\n      minimum: 0,\n      exclusive_minimum: true\n    )\n\n    # Multiple of 0.5\n    half_step_field = OpenAI.Responses.Schema.number(\n      multiple_of: 0.5\n    )","ref":"OpenAI.Responses.Schema.html#number/1-examples"},{"type":"function","title":"OpenAI.Responses.Schema.object/2","doc":"Creates an object schema definition from a map of field specifications.","ref":"OpenAI.Responses.Schema.html#object/2"},{"type":"function","title":"Parameters - OpenAI.Responses.Schema.object/2","doc":"* `fields` - A map where keys are field names and values are type specifications.\n    Type specifications can be:\n    * Atoms (`:string`, `:number`, `:integer`, `:boolean`) for simple types\n    * Tuples like `{:array, type}` for simple arrays\n    * Schema maps created by other Schema functions\n  * `opts` - Optional parameters\n    * `:name` - Name for the root schema","ref":"OpenAI.Responses.Schema.html#object/2-parameters"},{"type":"function","title":"Examples - OpenAI.Responses.Schema.object/2","doc":"# Simple object schema with atom types\n    schema = OpenAI.Responses.Schema.object(%{\n      name: :string,\n      date: :string,\n      participants: {:array, :string}\n    }, name: \"event\")\n\n    # Object schema with mixed simple and complex types\n    schema = OpenAI.Responses.Schema.object(%{\n      name: :string,\n      email: OpenAI.Responses.Schema.string(format: \"email\"),\n      age: :integer,\n      interests: {:array, :string}\n    })\n\n    # Use in a response\n    {:ok, response} = OpenAI.Responses.create(\"gpt-4.1\", \"Extract the event...\",\n      response_format: %{type: \"json_schema\", schema: schema}\n    )","ref":"OpenAI.Responses.Schema.html#object/2-examples"},{"type":"function","title":"OpenAI.Responses.Schema.string/1","doc":"Creates a string schema definition with optional constraints.\n\nFor a simple string without constraints, you can use the atom `:string` directly\nin `object/2` or `array/2` instead of calling this function.","ref":"OpenAI.Responses.Schema.html#string/1"},{"type":"function","title":"Parameters - OpenAI.Responses.Schema.string/1","doc":"* `opts` - Optional parameters\n    * `:format` - String format (e.g., \"date-time\", \"email\")\n    * `:pattern` - Regex pattern\n    * `:min_length` - Minimum length\n    * `:max_length` - Maximum length\n    * `:enum` - List of allowed values","ref":"OpenAI.Responses.Schema.html#string/1-parameters"},{"type":"function","title":"Examples - OpenAI.Responses.Schema.string/1","doc":"# Simple string (in object definition)\n    name_field = :string\n\n    # Email string\n    email_field = OpenAI.Responses.Schema.string(format: \"email\")\n\n    # String with length constraints\n    username_field = OpenAI.Responses.Schema.string(\n      min_length: 3,\n      max_length: 20\n    )\n\n    # String with allowed values (enum)\n    status_field = OpenAI.Responses.Schema.string(\n      enum: [\"pending\", \"active\", \"completed\"]\n    )","ref":"OpenAI.Responses.Schema.html#string/1-examples"},{"type":"module","title":"OpenAI.Responses.Client","doc":"HTTP client for the OpenAI Responses API.\n\nThis module handles the communication with OpenAI's API,\nincluding authentication, request formatting, and response parsing.","ref":"OpenAI.Responses.Client.html"},{"type":"function","title":"OpenAI.Responses.Client.new/1","doc":"Creates a new API client.","ref":"OpenAI.Responses.Client.html#new/1"},{"type":"function","title":"Options - OpenAI.Responses.Client.new/1","doc":"* `:api_key` - Your OpenAI API key (overrides environment variable)\n  * `:api_base` - The base URL for API requests (default: \"https://api.openai.com/v1\")\n  * `:req_options` - Additional options to pass to Req\n  * `:request_logger` - Optional 1-arity function to log requests. Receives the `Req.Request` struct before the request is sent.","ref":"OpenAI.Responses.Client.html#new/1-options"},{"type":"function","title":"OpenAI.Responses.Client.request/5","doc":"Makes a request to the OpenAI API.","ref":"OpenAI.Responses.Client.html#request/5"},{"type":"function","title":"Parameters - OpenAI.Responses.Client.request/5","doc":"* `client` - The client from `new/1`\n  * `method` - The HTTP method (:get, :post, :delete, etc.)\n  * `path` - The API path (e.g., \"/responses\")\n  * `body` - The request body (for POST, PUT, etc.)\n  * `query` - Query parameters","ref":"OpenAI.Responses.Client.html#request/5-parameters"},{"type":"function","title":"Returns - OpenAI.Responses.Client.request/5","doc":"* `{:ok, response}` - On success\n  * `{:error, error}` - On failure","ref":"OpenAI.Responses.Client.html#request/5-returns"},{"type":"function","title":"OpenAI.Responses.Client.stream/3","doc":"Streams a response from the OpenAI API.\n\nUses Req's built-in streaming capabilities. This implementation\ncreates a proper Elixir Stream that processes events as they\narrive from the API.","ref":"OpenAI.Responses.Client.html#stream/3"},{"type":"function","title":"Parameters - OpenAI.Responses.Client.stream/3","doc":"* `client` - The client from `new/1`\n  * `path` - The API path (e.g., \"/responses\")\n  * `body` - The request body","ref":"OpenAI.Responses.Client.html#stream/3-parameters"},{"type":"function","title":"Returns - OpenAI.Responses.Client.stream/3","doc":"* A stream of events that can be processed in real-time","ref":"OpenAI.Responses.Client.html#stream/3-returns"},{"type":"module","title":"OpenAI.Responses.Helpers","doc":"Helper functions for working with OpenAI Responses.\n\nThis module provides utility functions for common tasks when working\nwith the OpenAI Responses API, including:\n\n- Extracting text and data from responses\n- Creating structured messages with images\n- Handling response status and errors\n\nSee the individual function documentation for usage examples.","ref":"OpenAI.Responses.Helpers.html"},{"type":"function","title":"OpenAI.Responses.Helpers.calculate_cost/1","doc":"Calculates the estimated cost from a raw OpenAI API response map.\n\nExtracts the model name and usage information from the response map\nand calls `calculate_cost/2`.","ref":"OpenAI.Responses.Helpers.html#calculate_cost/1"},{"type":"function","title":"Parameters - OpenAI.Responses.Helpers.calculate_cost/1","doc":"* `raw_response` (map) - The raw map returned by the OpenAI API client (e.g., Req).\n    Expected format: `%{ \"model\" => string, \"usage\" => map, ... }`","ref":"OpenAI.Responses.Helpers.html#calculate_cost/1-parameters"},{"type":"function","title":"Returns - OpenAI.Responses.Helpers.calculate_cost/1","doc":"* `{:ok, Decimal.t()}` - The calculated cost as a Decimal.\n  * `{:error, :unknown_model_name}` - If the model name is not found in the pricing table.\n  * `{:error, :invalid_response_format}` - If the `raw_response` map doesn't contain \"model\" or \"usage\".\n  * `{:error, :invalid_usage_format}` - If the nested \"usage\" map has an unexpected format.","ref":"OpenAI.Responses.Helpers.html#calculate_cost/1-returns"},{"type":"function","title":"Examples - OpenAI.Responses.Helpers.calculate_cost/1","doc":"iex> raw_response = %{\n    ...>   \"model\" => \"gpt-4o-mini\",\n    ...>   \"usage\" => %{ \"input_tokens\" => 500, \"input_tokens_details\" => %{\"cached_tokens\" => 100}, \"output_tokens\" => 1000 },\n    ...>   \"output\" => [...]\n    ...> }\n    iex> OpenAI.Responses.Helpers.calculate_cost(raw_response)\n    {:ok, #Decimal<0.0006675>}\n\n    iex> raw_response = %{\"model\" => \"unknown-model\", \"usage\" => %{ \"input_tokens\" => 100, \"input_tokens_details\" => %{\"cached_tokens\" => 0}, \"output_tokens\" => 50 }}\n    iex> OpenAI.Responses.Helpers.calculate_cost(raw_response)\n    {:error, :unknown_model_name}\n\n    iex> raw_response = %{\"model\" => \"gpt-4o\", \"usage\" => %{\"input_tokens\" => 10}} # Incomplete usage\n    iex> OpenAI.Responses.Helpers.calculate_cost(raw_response)\n    {:error, :invalid_usage_format}","ref":"OpenAI.Responses.Helpers.html#calculate_cost/1-examples"},{"type":"function","title":"OpenAI.Responses.Helpers.calculate_cost/2","doc":"Calculates the estimated cost of an OpenAI API call based on token usage.\n\nUses the provided model name and token usage map to calculate the cost.\nHandles model aliases (e.g., `gpt-4o` maps to `gpt-4o-2024-08-06`).\nApplies discounts for cached input tokens where applicable.\n\nCosts are based on the pricing defined in `@model_pricing` (per 1 million tokens),\nwhich uses the full, dated model names as keys.","ref":"OpenAI.Responses.Helpers.html#calculate_cost/2"},{"type":"function","title":"Parameters - OpenAI.Responses.Helpers.calculate_cost/2","doc":"* `model_name` (string) - The name of the model used (e.g., \"gpt-4o\", \"gpt-4o-mini-2024-07-18\").\n  * `token_usage` (map) - The token usage map, typically from `response[\"usage\"]`.\n    Expected format: `%{ \"input_tokens\" => integer, \"input_tokens_details\" => %{\"cached_tokens\" => integer}, \"output_tokens\" => integer, ... }`","ref":"OpenAI.Responses.Helpers.html#calculate_cost/2-parameters"},{"type":"function","title":"Returns - OpenAI.Responses.Helpers.calculate_cost/2","doc":"* `{:ok, Decimal.t()}` - The calculated cost as a Decimal.\n  * `{:error, :unknown_model_name}` - If the model name is not found in the pricing table.\n  * `{:error, :invalid_usage_format}` - If the `token_usage` map has an unexpected format.","ref":"OpenAI.Responses.Helpers.html#calculate_cost/2-returns"},{"type":"function","title":"Examples - OpenAI.Responses.Helpers.calculate_cost/2","doc":"iex> usage = %{ \"input_tokens\" => 1000, \"input_tokens_details\" => %{\"cached_tokens\" => 500}, \"output_tokens\" => 2000 }\n    iex> OpenAI.Responses.Helpers.calculate_cost(\"gpt-4o\", usage) # Using alias\n    {:ok, #Decimal<0.021875>}\n\n    iex> usage = %{ \"input_tokens\" => 1000, \"input_tokens_details\" => %{\"cached_tokens\" => 0}, \"output_tokens\" => 2000 }\n    iex> OpenAI.Responses.Helpers.calculate_cost(\"gpt-4o-2024-08-06\", usage) # Using full name\n    {:ok, #Decimal<0.0225>}\n\n    iex> usage = %{ \"input_tokens\" => 1000, \"input_tokens_details\" => %{\"cached_tokens\" => 500}, \"output_tokens\" => 2000 }\n    iex> OpenAI.Responses.Helpers.calculate_cost(\"gpt-4o-audio-preview-2024-12-17\", usage) # Model without cached input discount\n    {:ok, #Decimal<0.0225>}\n\n    iex> usage = %{ \"input_tokens\" => 100, \"input_tokens_details\" => %{\"cached_tokens\" => 0}, \"output_tokens\" => 50 }\n    iex> OpenAI.Responses.Helpers.calculate_cost(\"unknown-model\", usage)\n    {:error, :unknown_model_name}","ref":"OpenAI.Responses.Helpers.html#calculate_cost/2-examples"},{"type":"function","title":"OpenAI.Responses.Helpers.create_message_with_images/3","doc":"Creates a structured message with text and optional images.\n\nUses the ImageHelpers module to create a properly formatted message\nwith text and images for the OpenAI Responses API. See `OpenAI.Responses.Helpers.ImageHelpers.create_message_with_images/3`\nfor full documentation.","ref":"OpenAI.Responses.Helpers.html#create_message_with_images/3"},{"type":"function","title":"Examples - OpenAI.Responses.Helpers.create_message_with_images/3","doc":"# Simple text with one image URL\n    iex> input_message = OpenAI.Responses.Helpers.create_message_with_images(\"What is in this image?\", \"https://example.com/image.jpg\")\n    iex> {:ok, response} = OpenAI.Responses.create(\"gpt-4o\", [input_message])\n\n    # With a local file\n    iex> input_message = OpenAI.Responses.Helpers.create_message_with_images(\"Describe this\", \"/path/to/image.jpg\")\n\n    # With multiple images and high detail\n    iex> input_message = OpenAI.Responses.Helpers.create_message_with_images(\n    ...>   \"Compare these\",\n    ...>   [\"image1.jpg\", \"image2.jpg\"],\n    ...>   detail: \"high\"\n    ...> )","ref":"OpenAI.Responses.Helpers.html#create_message_with_images/3-examples"},{"type":"function","title":"OpenAI.Responses.Helpers.has_refusal?/1","doc":"Checks if a response contains a refusal.","ref":"OpenAI.Responses.Helpers.html#has_refusal?/1"},{"type":"function","title":"Parameters - OpenAI.Responses.Helpers.has_refusal?/1","doc":"* `response` - The response from OpenAI.Responses.create/3","ref":"OpenAI.Responses.Helpers.html#has_refusal?/1-parameters"},{"type":"function","title":"Returns - OpenAI.Responses.Helpers.has_refusal?/1","doc":"* true if the response contains a refusal, false otherwise","ref":"OpenAI.Responses.Helpers.html#has_refusal?/1-returns"},{"type":"function","title":"OpenAI.Responses.Helpers.output_text/1","doc":"Extracts the output text from a response.\n\nThis function navigates the response structure to find and return the generated text.\nIf the response contains multiple outputs or messages, it concatenates them with newlines.","ref":"OpenAI.Responses.Helpers.html#output_text/1"},{"type":"function","title":"Parameters - OpenAI.Responses.Helpers.output_text/1","doc":"* `response` - The response from OpenAI.Responses.create/3","ref":"OpenAI.Responses.Helpers.html#output_text/1-parameters"},{"type":"function","title":"Returns - OpenAI.Responses.Helpers.output_text/1","doc":"* The extracted text as a string\n  * An empty string if no text output is found","ref":"OpenAI.Responses.Helpers.html#output_text/1-returns"},{"type":"function","title":"Examples - OpenAI.Responses.Helpers.output_text/1","doc":"{:ok, response} = OpenAI.Responses.create(\"gpt-4o\", \"Write a haiku about programming\")\n    text = OpenAI.Responses.Helpers.output_text(response)\n    # => \"Fingers tap keyboard\nLogic blooms in silent code\nBugs hide in plain sight\"","ref":"OpenAI.Responses.Helpers.html#output_text/1-examples"},{"type":"function","title":"OpenAI.Responses.Helpers.refusal_message/1","doc":"Gets the refusal message if one exists.","ref":"OpenAI.Responses.Helpers.html#refusal_message/1"},{"type":"function","title":"Parameters - OpenAI.Responses.Helpers.refusal_message/1","doc":"* `response` - The response from OpenAI.Responses.create/3","ref":"OpenAI.Responses.Helpers.html#refusal_message/1-parameters"},{"type":"function","title":"Returns - OpenAI.Responses.Helpers.refusal_message/1","doc":"* The refusal message as a string\n  * nil if no refusal is found","ref":"OpenAI.Responses.Helpers.html#refusal_message/1-returns"},{"type":"function","title":"OpenAI.Responses.Helpers.status/1","doc":"Extracts the status from a response.","ref":"OpenAI.Responses.Helpers.html#status/1"},{"type":"function","title":"Parameters - OpenAI.Responses.Helpers.status/1","doc":"* `response` - The response from OpenAI.Responses.create/3","ref":"OpenAI.Responses.Helpers.html#status/1-parameters"},{"type":"function","title":"Returns - OpenAI.Responses.Helpers.status/1","doc":"* The status as a string (e.g., \"completed\", \"in_progress\")\n  * nil if no status is found","ref":"OpenAI.Responses.Helpers.html#status/1-returns"},{"type":"function","title":"OpenAI.Responses.Helpers.token_usage/1","doc":"Gets token usage information from a response.","ref":"OpenAI.Responses.Helpers.html#token_usage/1"},{"type":"function","title":"Parameters - OpenAI.Responses.Helpers.token_usage/1","doc":"* `response` - The response from OpenAI.Responses.create/3","ref":"OpenAI.Responses.Helpers.html#token_usage/1-parameters"},{"type":"function","title":"Returns - OpenAI.Responses.Helpers.token_usage/1","doc":"* A map with token usage information\n  * nil if no usage information is found","ref":"OpenAI.Responses.Helpers.html#token_usage/1-returns"},{"type":"function","title":"Examples - OpenAI.Responses.Helpers.token_usage/1","doc":"{:ok, response} = OpenAI.Responses.create(\"gpt-4o\", \"Write a haiku about programming\")\n    usage = OpenAI.Responses.Helpers.token_usage(response)\n    # => %{\"input_tokens\" => 8, \"output_tokens\" => 17, \"total_tokens\" => 25}","ref":"OpenAI.Responses.Helpers.html#token_usage/1-examples"},{"type":"module","title":"OpenAI.Responses.Stream","doc":"Utilities for working with OpenAI streaming responses.\n\nThis module provides functions for transforming and consuming\nstreamed responses from the OpenAI API.","ref":"OpenAI.Responses.Stream.html"},{"type":"module","title":"Examples - OpenAI.Responses.Stream","doc":"# Get text deltas as they arrive\n    stream = OpenAI.Responses.stream(\"gpt-4.1\", \"Tell me a story\")\n    text_stream = OpenAI.Responses.Stream.text_deltas(stream)\n\n    # Print each text delta in real-time without duplication\n    text_stream\n    |> Stream.each(fn delta ->\n      IO.write(delta)\n    end)\n    |> Stream.run()\n\n    # Collect a complete response from a stream\n    stream = OpenAI.Responses.stream(\"gpt-4.1\", \"Tell me a story\")\n    response = OpenAI.Responses.Stream.collect(stream)","ref":"OpenAI.Responses.Stream.html#module-examples"},{"type":"function","title":"OpenAI.Responses.Stream.collect/1","doc":"Collects all streaming events into a final response.\n\nThis is useful for consuming a stream and building a complete response object,\nsimilar to what would be returned by a non-streaming API call. The function\nprocesses all events from the stream and combines them into a single coherent\nresponse object.","ref":"OpenAI.Responses.Stream.html#collect/1"},{"type":"function","title":"Parameters - OpenAI.Responses.Stream.collect/1","doc":"* `stream` - The stream from OpenAI.Responses.stream/3","ref":"OpenAI.Responses.Stream.html#collect/1-parameters"},{"type":"function","title":"Returns - OpenAI.Responses.Stream.collect/1","doc":"* The complete response map","ref":"OpenAI.Responses.Stream.html#collect/1-returns"},{"type":"function","title":"Examples - OpenAI.Responses.Stream.collect/1","doc":"# Get a streaming response\n    stream = OpenAI.Responses.stream(\"gpt-4.1\", \"Tell me a story\")\n\n    # Collect all events into a single response object\n    response = OpenAI.Responses.Stream.collect(stream)\n\n    # Process the complete response\n    text = OpenAI.Responses.Helpers.output_text(response)\n    IO.puts(text)","ref":"OpenAI.Responses.Stream.html#collect/1-examples"},{"type":"function","title":"OpenAI.Responses.Stream.new/2","doc":"Creates a new stream handler for OpenAI streaming responses.\n\nThis function is maintained for backward compatibility.\nFor new code, use the stream transformation functions directly.","ref":"OpenAI.Responses.Stream.html#new/2"},{"type":"function","title":"Parameters - OpenAI.Responses.Stream.new/2","doc":"* `stream` - The stream from OpenAI.Responses.stream/3\n  * `opts` - Options for the stream handler","ref":"OpenAI.Responses.Stream.html#new/2-parameters"},{"type":"function","title":"Returns - OpenAI.Responses.Stream.new/2","doc":"* A stream handler struct","ref":"OpenAI.Responses.Stream.html#new/2-returns"},{"type":"function","title":"OpenAI.Responses.Stream.text_chunks/1","doc":"Extracts text chunks from a stream as they arrive (legacy function).\n\nThis function is maintained for backward compatibility.\nFor new code, use `text_deltas/1` instead.","ref":"OpenAI.Responses.Stream.html#text_chunks/1"},{"type":"function","title":"Parameters - OpenAI.Responses.Stream.text_chunks/1","doc":"* `stream_handler` - The stream handler from new/2","ref":"OpenAI.Responses.Stream.html#text_chunks/1-parameters"},{"type":"function","title":"Returns - OpenAI.Responses.Stream.text_chunks/1","doc":"* A stream of text chunks","ref":"OpenAI.Responses.Stream.html#text_chunks/1-returns"},{"type":"function","title":"OpenAI.Responses.Stream.text_deltas/1","doc":"Extracts text deltas from a stream as they arrive.\n\nThis returns a stream of text chunks that can be consumed as they arrive,\nrather than waiting for the full response. The implementation avoids duplicating\ncontent when the API sends both incremental deltas and a final complete text.","ref":"OpenAI.Responses.Stream.html#text_deltas/1"},{"type":"function","title":"Parameters - OpenAI.Responses.Stream.text_deltas/1","doc":"* `stream` - The stream from OpenAI.Responses.stream/3","ref":"OpenAI.Responses.Stream.html#text_deltas/1-parameters"},{"type":"function","title":"Returns - OpenAI.Responses.Stream.text_deltas/1","doc":"* A stream of text chunks","ref":"OpenAI.Responses.Stream.html#text_deltas/1-returns"},{"type":"function","title":"Examples - OpenAI.Responses.Stream.text_deltas/1","doc":"stream = OpenAI.Responses.stream(\"gpt-4.1\", \"Tell me a story\")\n    text_stream = OpenAI.Responses.Stream.text_deltas(stream)\n\n    # Print text deltas as they arrive (real-time output)\n    text_stream\n    |> Stream.each(fn delta ->\n      IO.write(delta)\n    end)\n    |> Stream.run()\n    IO.puts(\"\")  # Add a newline at the end","ref":"OpenAI.Responses.Stream.html#text_deltas/1-examples"},{"type":"function","title":"OpenAI.Responses.Stream.transform/2","doc":"Transforms a stream with a callback function.","ref":"OpenAI.Responses.Stream.html#transform/2"},{"type":"function","title":"Parameters - OpenAI.Responses.Stream.transform/2","doc":"* `stream_handler` - The stream handler from new/2\n  * `callback` - The callback function to apply to each event","ref":"OpenAI.Responses.Stream.html#transform/2-parameters"},{"type":"function","title":"Returns - OpenAI.Responses.Stream.transform/2","doc":"* A transformed stream","ref":"OpenAI.Responses.Stream.html#transform/2-returns"},{"type":"extras","title":"Basic Usage","doc":"# Basic Usage\n\nA simple Elixir client for the OpenAI Responses API, built on top of\n[Req](https://github.com/wojtekmach/req).\n\n> **[Interactive LiveBook Tutorial](notebooks/tutorial.livemd)** - Learn how to use this library with interactive examples","ref":"readme.html"},{"type":"extras","title":"Installation - Basic Usage","doc":"Add `openai_responses` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:openai_responses, \"~> 0.3.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Configuration - Basic Usage","doc":"Set your OpenAI API key using one of these methods:\n\n```elixir\n# 1. Using environment variable (recommended)\nSystem.put_env(\"OPENAI_API_KEY\", \"your-api-key\")\n\n# 2. Passing directly to functions\nOpenAI.Responses.create(model: \"gpt-4.1\", prompt: \"Hello\", api_key: \"your-api-key\")\n\n# 3. Creating a custom client\nclient = OpenAI.Responses.Client.new(api_key: \"your-api-key\")\nOpenAI.Responses.create(model: \"gpt-4.1\", prompt: \"Hello\", client: client)\n```","ref":"readme.html#configuration"},{"type":"extras","title":"Basic Usage - Basic Usage","doc":"","ref":"readme.html#basic-usage"},{"type":"extras","title":"Creating a Simple Response - Basic Usage","doc":"```elixir\n{:ok, response} = OpenAI.Responses.create(model: \"gpt-4.1\", prompt: \"Write a haiku about programming\")\n\n# Extract the text output\ntext = OpenAI.Responses.Helpers.output_text(response)\nIO.puts(text)\n\n# Check token usage\ntoken_usage = OpenAI.Responses.Helpers.token_usage(response)\nIO.inspect(token_usage)\n```","ref":"readme.html#creating-a-simple-response"},{"type":"extras","title":"Using Tools and Options - Basic Usage","doc":"```elixir\n{:ok, response} = OpenAI.Responses.create(\n  model: \"gpt-4.1\",\n  prompt: \"What's the weather like in Paris?\",\n  tools: [%{type: \"web_search_preview\"}],\n  temperature: 0.7\n)\n\ntext = OpenAI.Responses.Helpers.output_text(response)\nIO.puts(text)\n```","ref":"readme.html#using-tools-and-options"},{"type":"extras","title":"Structured Input - Basic Usage","doc":"```elixir\n# Create a structured input with helper function\ninput_message = OpenAI.Responses.Helpers.create_input_message(\n  \"What is in this image?\",\n  \"https://example.com/image.jpg\"\n)\n\n{:ok, response} = OpenAI.Responses.create(model: \"gpt-4.1\", input: [input_message])\n\n# With local images (automatically encoded to Base64)\ninput_message = OpenAI.Responses.Helpers.create_input_message(\n  \"Describe these images\",\n  [\"path/to/image1.jpg\", \"path/to/image2.jpg\"],\n  detail: \"high\"  # Optional detail level\n)\n\n{:ok, response} = OpenAI.Responses.create(model: \"gpt-4.1\", input: [input_message])\n\n# Or manually create the structured input\ninput = [\n  %{\n    \"role\" => \"user\",\n    \"content\" => [\n      %{\"type\" => \"input_text\", \"text\" => \"What is in this image?\"},\n      %{\n        \"type\" => \"input_image\",\n        \"image_url\" => \"https://example.com/image.jpg\"\n      }\n    ]\n  }\n]\n\n{:ok, response} = OpenAI.Responses.create(model: \"gpt-4.1\", input: input)\n```","ref":"readme.html#structured-input"},{"type":"extras","title":"Streaming Responses - Basic Usage","doc":"The library now uses Req's built-in streaming capabilities with the `:into`\nkeyword for more reliable streaming.\n\n```elixir\n# Get a stream of events (returns an Enumerable)\nstream = OpenAI.Responses.stream(model: \"gpt-4.1\", prompt: \"Tell me a story\")\n\n# Iterate over raw events as they arrive (true streaming)\nstream\n|> Stream.each(&IO.inspect/1)\n|> Stream.run()\n\n# Print text deltas as they arrive (real-time output)\nstream = OpenAI.Responses.stream(model: \"gpt-4.1\", prompt: \"Tell me a story\")\ntext_stream = OpenAI.Responses.text_deltas(stream)\n\n# This preserves streaming behavior (one chunk at a time)\ntext_stream\n|> Stream.each(fn delta ->\n  IO.write(delta)\nend)\n|> Stream.run()\nIO.puts(\"\")   # Add a newline at the end\n\n# Create a typing effect\nstream = OpenAI.Responses.stream(model: \"gpt-4.1\", prompt: \"Tell me a story\")\ntext_stream = OpenAI.Responses.text_deltas(stream)\n\ntext_stream\n|> Stream.each(fn delta ->\n  IO.write(delta)\n  Process.sleep(10)  # Add delay for typing effect\nend)\n|> Stream.run()\nIO.puts(\"\")\n\n# Collect a complete response from a stream\nstream = OpenAI.Responses.stream(model: \"gpt-4.1\", prompt: \"Tell me a story\")\nresponse = OpenAI.Responses.collect_stream(stream)\n\n# Work with the collected response\ntext = OpenAI.Responses.Helpers.output_text(response)\nIO.puts(text)\n```","ref":"readme.html#streaming-responses"},{"type":"extras","title":"Other Operations - Basic Usage","doc":"```elixir\n# Get a specific response by ID\n{:ok, response} = OpenAI.Responses.get(\"resp_123\")\n\n# Delete a response\n{:ok, result} = OpenAI.Responses.delete(\"resp_123\")\n\n# List input items for a response\n{:ok, items} = OpenAI.Responses.list_input_items(\"resp_123\")\n```","ref":"readme.html#other-operations"},{"type":"extras","title":"Helper Functions - Basic Usage","doc":"The `OpenAI.Responses.Helpers` module provides utility functions for working\nwith responses:\n\n```elixir\n# Extract text from a response\ntext = OpenAI.Responses.Helpers.output_text(response)\n\n# Get token usage information\nusage = OpenAI.Responses.Helpers.token_usage(response)\n\n# Check response status\nstatus = OpenAI.Responses.Helpers.status(response)\n\n# Check for refusal\nif OpenAI.Responses.Helpers.has_refusal?(response) do\n  refusal_message = OpenAI.Responses.Helpers.refusal_message(response)\n  IO.puts(\"Request was refused: #{refusal_message}\")\nend\n```","ref":"readme.html#helper-functions"},{"type":"extras","title":"Documentation - Basic Usage","doc":"For more detailed documentation, run `mix docs` to generate full API\ndocumentation.","ref":"readme.html#documentation"},{"type":"extras","title":"License - Basic Usage","doc":"MIT","ref":"readme.html#license"},{"type":"extras","title":"Structured Outputs","doc":"# Structured Outputs\n\nThis document explains how to use the structured outputs feature with the OpenAI Responses Elixir library.","ref":"structured_output.html"},{"type":"extras","title":"Introduction - Structured Outputs","doc":"Structured Outputs is a feature that ensures the model will always generate responses that adhere to your supplied JSON Schema, so you don't need to worry about the model omitting a required key, or hallucinating an invalid enum value.\n\nThe OpenAI Responses Elixir library provides a simple, idiomatic way to define schemas and parse responses with structured outputs. This library supports the latest OpenAI API format for structured outputs.","ref":"structured_output.html#introduction"},{"type":"extras","title":"Basic Usage - Structured Outputs","doc":"```elixir\nalias OpenAI.Responses\nalias OpenAI.Responses.Schema\n\n# Define a schema\ncalendar_event_schema = Schema.object(%{\n  name: :string,\n  date: :string,\n  participants: {:array, :string}\n})\n\n# Create a response with structured output\n{:ok, result} = Responses.parse(\n  calendar_event_schema,\n  model: \"gpt-4.1\",\n  input: \"Alice and Bob are going to a science fair on Friday.\",\n  schema_name: \"event\"\n)\n\n# Access the parsed data\nIO.puts(\"Event: #{result.parsed[\"name\"]} on #{result.parsed[\"date\"]}\")\nIO.puts(\"Participants: #{Enum.join(result.parsed[\"participants\"], \", \")}\")\nIO.inspect(result.token_usage, label: \"Token Usage\")\n```","ref":"structured_output.html#basic-usage"},{"type":"extras","title":"Defining Schemas - Structured Outputs","doc":"","ref":"structured_output.html#defining-schemas"},{"type":"extras","title":"Simple Types - Structured Outputs","doc":"For simple types without constraints, you can use atoms directly:\n\n```elixir\nschema = Schema.object(%{\n  name: :string,\n  age: :integer,\n  is_active: :boolean,\n  scores: {:array, :number}\n})\n```","ref":"structured_output.html#simple-types"},{"type":"extras","title":"Types with Constraints - Structured Outputs","doc":"For types with constraints or additional options, use the corresponding functions:\n\n```elixir\nschema = Schema.object(%{\n  username: Schema.string(min_length: 3, max_length: 20),\n  rating: Schema.number(minimum: 0, maximum: 5),\n  tags: Schema.array(:string, min_items: 1, max_items: 5)\n})\n```","ref":"structured_output.html#types-with-constraints"},{"type":"extras","title":"Nested Objects - Structured Outputs","doc":"You can define nested objects:\n\n```elixir\nschema = Schema.object(%{\n  user: Schema.object(%{\n    name: :string,\n    email: Schema.string(format: \"email\")\n  }),\n  preferences: Schema.object(%{\n    theme: :string,\n    notifications: :boolean\n  })\n})\n```","ref":"structured_output.html#nested-objects"},{"type":"extras","title":"Arrays - Structured Outputs","doc":"Arrays can be defined in two ways:\n\n```elixir\n# Simple array of strings\ntags_schema = {:array, :string}\n\n# Array with constraints\ntags_schema = Schema.array(:string, min_items: 1, max_items: 5)\n\n# Array of objects\nsteps_schema = Schema.array(\n  Schema.object(%{\n    explanation: :string,\n    output: :string\n  })\n)\n```","ref":"structured_output.html#arrays"},{"type":"extras","title":"Nullable Fields - Structured Outputs","doc":"You can make fields nullable:\n\n```elixir\nschema = Schema.object(%{\n  middle_name: Schema.nullable(:string),\n  address: Schema.nullable(\n    Schema.object(%{\n      street: :string,\n      city: :string,\n      zip: :string\n    })\n  )\n})\n```","ref":"structured_output.html#nullable-fields"},{"type":"extras","title":"Parsing Responses - Structured Outputs","doc":"","ref":"structured_output.html#parsing-responses"},{"type":"extras","title":"Regular Parsing - Structured Outputs","doc":"Use the `parse/2` function with options to get a structured response:\n\n```elixir\n{:ok, result} = Responses.parse(\n  my_schema,\n  model: \"gpt-4.1\",\n  input: \"Extract information from this text...\"\n)\n# Access parsed data with result.parsed\n```","ref":"structured_output.html#regular-parsing"},{"type":"extras","title":"Examples - Structured Outputs","doc":"See the `examples/structured_output_example.exs` file for complete examples of using structured outputs.","ref":"structured_output.html#examples"},{"type":"extras","title":"Supported Models - Structured Outputs","doc":"Structured Outputs is available in OpenAI's latest large language models, starting with gpt-4.1:\n\n- `gpt-4.5-preview-2025-02-27` and later\n- `o3-mini-2025-1-31` and later\n- `o1-2024-12-17` and later\n- `gpt-4.1-mini-2024-07-18` and later\n- `gpt-4.1-2024-08-06` and later\n\nOlder models like `gpt-4-turbo` and earlier may use JSON mode instead.","ref":"structured_output.html#supported-models"},{"type":"extras","title":"API Implementation Details - Structured Outputs","doc":"This library implements the latest OpenAI API format for structured outputs, which uses the `text` parameter with a format specification instead of the older `response_format` parameter. The implementation handles both the direct API response format and streaming responses.\n\nKey features of the implementation:\n\n- Automatic handling of the new API format with the `text` parameter\n- Support for schema validation with `additionalProperties: false` by default\n- Proper extraction of structured data from both regular and streaming responses\n- Comprehensive error handling for JSON parsing and schema validation","ref":"structured_output.html#api-implementation-details"},{"type":"extras","title":"Tutorial","doc":"# Tutorial\n\n```elixir\nMix.install([\n  {:openai_responses, \"~> 0.3.0\"},\n  # {:openai_responses, path: \"~/src/openai_responses\"},\n  {:kino, \"~> 0.11.0\"}\n])\n```","ref":"tutorial.html"},{"type":"extras","title":"Introduction - Tutorial","doc":"The only setup you need for using the library is to get your OpenAI API token. If you already have the `OPENAI_API_KEY` environment variable set, then you can start right away.\n\n```elixir\nalias OpenAI.Responses\nalias OpenAI.Responses.Helpers\n```\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"tutorial.html#introduction"},{"type":"extras","title":"Basic usage - Tutorial","doc":"`create/1` requires a keyword list with at least two arguments: the model name and the input text:\n\n```elixir\n{:ok, response} = Responses.create(model: \"gpt-4.1-nano\", input: \"Write a haiku about programming\")\n```\n\nThe `response` is just a map, and you can use helper functions to extract information from it:\n\n```elixir\nHelpers.has_refusal?(response)\n```\n\n```elixir\nHelpers.output_text(response)\n```\n\n```elixir\nHelpers.token_usage(response)\n```\n\n```elixir\nHelpers.calculate_cost(response)\n```\n\nYou can also supply additional parameters to the API call using the keyword list:\n\n```elixir\n{:ok, response} =\n  Responses.create(\n    model: \"gpt-4.1-nano\",\n    input: \"Do you need semicolons in Elixir\",\n    instructions: \"Talk like a pirate\"\n  )\nIO.puts Helpers.output_text(response)\n```\n\nA *structured* input can be manually constructed and passed to `create/1`:\n\n```elixir\n{:ok, response} =\n  Responses.create(\n    model: \"gpt-4.1-nano\",\n    input: [\n      %{role: \"user\", content: \"knock knock.\"},\n      %{role: \"assistant\", content: \"Who's there?\"},\n      %{role: \"user\", content: \"Orange.\"}\n    ]\n  )\nIO.puts Helpers.output_text(response)\n```\n\n```elixir\ninput = [\n  %{\n    \"role\" => \"user\",\n    \"content\" => [\n      %{\"type\" => \"input_text\", \"text\" => \"What is in this image?\"},\n      %{\n        \"type\" => \"input_image\",\n        \"image_url\" => \"https://upload.wikimedia.org/wikipedia/commons/d/d2/Three_early_medicine_bottles.jpg\"\n      }\n    ]\n  }\n]\n\n{:ok, response} = OpenAI.Responses.create(model: \"gpt-4.1-nano\", input: input)\nIO.puts Helpers.output_text(response)\n```\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"tutorial.html#basic-usage"},{"type":"extras","title":"Image helpers - Tutorial","doc":"As we saw in the previous section, you can manually create a structured input with images, but this requires writing verbose JSON-like structures. The library provides helper functions to make this process more ergonomic.\n\n```elixir\n# Using the helper function to create a message with an image\ninput_message = Helpers.create_message_with_images(\n  \"What is in this image?\", \n  \"https://upload.wikimedia.org/wikipedia/commons/d/d2/Three_early_medicine_bottles.jpg\"\n)\n\n# The helper creates the same structure as the manual approach, but with less code\ninput_message\n```\n\nYou can also specify multiple images with different detail levels:\n\n```elixir\nmulti_image_message = Helpers.create_message_with_images(\n  \"Compare these two images\",\n  [\n    {\"https://upload.wikimedia.org/wikipedia/commons/d/d2/Three_early_medicine_bottles.jpg\", \"high\"},\n    \"https://upload.wikimedia.org/wikipedia/commons/4/48/Cocacolacollection.JPG\"\n  ],\n  detail: \"low\"  # Default detail level for images without a specific level\n)\n\n# And then use it with the API\n{:ok, response} = OpenAI.Responses.create(model: \"gpt-4.1-nano\", input: [multi_image_message])\nIO.puts Helpers.output_text(response)\n```\n\nLocal image files are also supported and will be automatically encoded as base64 data URLs:\n\n```elixir\n# This would work if you have these image files locally\n# local_image_message = Helpers.create_message_with_images(\n#   \"Describe these local images\",\n#   [\"path/to/image1.jpg\", \"path/to/image2.png\"]\n# )\n```\n\nThe helper function eliminates boilerplate code, handles encoding of local images, and provides a more intuitive interface for working with images in your prompts.\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"tutorial.html#image-helpers"},{"type":"extras","title":"Using built-in tools - Tutorial","doc":"The usage of built-in tools can be illustrated by the following example:\n\n```elixir\n{:ok, response_no_tools} = Responses.create(model: \"gpt-4.1-nano\", input: \"What's the weather in San Francisco?\")\nIO.puts(Helpers.output_text(response_no_tools))\n```\n\n```elixir\n{:ok, response_with_search} =\n  Responses.create(model: \"gpt-4.1-mini\", input: \"What's the weather in San Francisco?\",\n    tools: [%{type: \"web_search_preview\"}],\n    temperature: 0.7\n  )\n\nIO.puts(Helpers.output_text(response_with_search))\n```\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"tutorial.html#using-built-in-tools"},{"type":"extras","title":"Stored conversation state - Tutorial","doc":"The new Responses API makes it much easier to continue the conversation, without repeating the whole conversation story. Note that responses **are** stored by default.\n\nOne thing to be mindful about: when using along with `previous_response_id`, the instructions from a previous response will not be carried over to the next response.\n\n```elixir\n{:ok, response} =\n  Responses.create(\n    model: \"gpt-4.1-nano\",\n    input: [\n      %{\n        role: \"developer\",\n        content: \"Talk like a pirate in all your responses, including all the interesting facts.\"\n      },\n      %{\n        role: \"user\",\n        content: \"\"\"\n        Give me the names of the first 6 U.S. presidents. Include their names, dates of birth and death,\n        and information on their terms. Provide some interesting facts about each\n        \"\"\"\n      }\n    ]\n  )\n\nHelpers.output_text(response) |> IO.puts()\n```\n\nNow we can get information about the next 6 presidents, without explicitly repeating our prompt, by refering to the `response[\"id\"]` from the previous response:\n\n```elixir\n{:ok, next_response} =\n  Responses.create(model: \"gpt-4.1-nano\", input: \"next 6\", previous_response_id: response[\"id\"])\n\nHelpers.output_text(next_response) |> IO.puts()\n```\n\n```elixir\n{:ok, next_response2} =\n  Responses.create(model: \"gpt-4.1-nano\", input: \"next 6\", previous_response_id: next_response[\"id\"])\n{:ok, next_response3} =\n  Responses.create(model: \"gpt-4.1-nano\", input: \"next 6\", previous_response_id: next_response2[\"id\"])\n```\n\nLet's compare the token usage of these two requests. As you can see, even though the *explicit* prompt we supplied was very short, OpenAI still takes into account all the relevant input tokens from the prompt. However, for system prompts over 1024 bytes, OpenAI enables caching automatically, which can be seen below:\n\n```elixir\nIO.puts inspect(Helpers.token_usage(response), pretty: true)\nIO.puts inspect(Helpers.token_usage(next_response), pretty: true)\nIO.puts inspect(Helpers.token_usage(next_response3), pretty: true)\n```\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"tutorial.html#stored-conversation-state"},{"type":"extras","title":"Structured outputs - Tutorial","doc":"Structured output ensures the model always generates responses that adhere to your supplied JSON Schema.\n\n```elixir\nalias OpenAI.Responses.Schema\n\n# Define a schema for a calendar event\ncalendar_event_schema = Schema.object(%{\n  name: :string,\n  date: :string,\n  participants: {:array, :string}\n})\n\n# Parse a response with structured output\n{:ok, result} = Responses.parse(\n  calendar_event_schema,\n  model: \"gpt-4.1-nano\",\n  input: \"Alice and Bob are going to a science fair on Friday.\",\n  schema_name: \"event\"\n)\n\nresult.parsed\n```\n\n```elixir\n# Define a more complex schema with nested objects\nmath_reasoning_schema = Schema.object(%{\n  steps: {:array, Schema.object(%{\n    explanation: :string,\n    output: :string\n  })},\n  final_answer: :string\n})\n\n# Parse with the complex schema\n{:ok, solution} = Responses.parse(\n  math_reasoning_schema,\n  model: \"gpt-4.1-mini\",\n  input: \"Solve 8x + 7 = -23\"\n)\n\nsolution.parsed\n```\n\nThe returned map also defines `:raw_response` and `:token_usage` if you need to get additional metadata from the LLM provider.\n\n```elixir\nsolution.raw_response\n```\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"tutorial.html#structured-outputs"},{"type":"extras","title":"Streaming responses - Tutorial","doc":"OpenAI.Responses supports true streaming, where you can process chunks as they arrive without waiting for the entire response to complete.\n\n<!-- livebook:{\"break_markdown\":true} -->","ref":"tutorial.html#streaming-responses"},{"type":"extras","title":"Real-time text streaming - Tutorial","doc":"This example demonstrates how to display text as it arrives in real-time using Kino.Frame:\n\n```elixir\nframe = Kino.Frame.new()\nKino.render(frame)\n\n# Create a stream from OpenAI\nstream = Responses.stream(model: \"gpt-4.1-mini\", input: \"Write a short poem about coding in Elixir\")\n\n# Extract text deltas - no need for initializing a stream handler\ntext_stream = Responses.text_deltas(stream)\n\nKino.Frame.append(frame, Kino.Markdown.new(\"","ref":"tutorial.html#real-time-text-streaming"},{"type":"extras","title":"Poem about coding\\n\")) - Tutorial","doc":"# Process the stream\ntext_stream\n|> Stream.each(fn delta ->\n  Kino.Frame.append(frame, Kino.Markdown.new(delta, chunk: true))\nend)\n|> Stream.run()\n\n\nKino.Frame.append(frame, Kino.Markdown.new(\"\\n\\n*Generation complete* ✨\"))\n\n:done\n```\n\n<!-- livebook:{\"offset\":8590,\"stamp\":{\"token\":\"XCP.5MGlElgFvRAvL_fxgd1CJi-9RMI-GmxolRpYtN1KlTICcwfvXlEJoQMzk8uqvJMWY-ddghg9DedlofaK5FJdPxOBAqZwBprIkJqFO31S2sJpDdKR1WIjs5I\",\"version\":2}} -->","ref":"tutorial.html#poem-about-coding-n"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}